#ifndef BatchLab_Annotation_h
#define BatchLab_Annotation_h

// Inheritance :
#include <AIDA/IAnnotation.h>
#include <Slash/Store/IStorable.h>

#include <BatchLab/Core/T_Vector.h>

namespace BatchLab {

class Item : public virtual Slash::Store::IStorable {
public: //IVisited
  virtual void* cast(const std::string&) const;
  virtual bool visit(Slash::Store::IConstVisitor&) const;
  virtual bool read(Slash::Store::IVisitor&);
public:
  Item();
  Item(const std::string& aKey,const std::string& aValue,bool aSticky);
  Item(const Item&);
  virtual ~Item();
  Item& operator=(const Item&);
public:
  std::string fKey;
  std::string fValue;
  bool fSticky;
  SLASH_STORE_HEADER
};

class Annotation 
:public virtual AIDA::IAnnotation 
,public virtual Slash::Store::IStorable {
public: //AIDA/IAnnotation
  virtual bool addItem(const std::string&,const std::string&,bool = true);
  virtual bool removeItem(const std::string&);
  virtual std::string value(const std::string&) const;
  virtual bool visibility(const std::string&) const;
  virtual void setValue(const std::string&,const std::string&);
  virtual void setSticky(const std::string&,bool);
  virtual int size() const;
  virtual std::string key(int) const;
  virtual std::string value(int) const;
  virtual void reset();
  //AIDA-v3r3p0
  virtual bool isSticky(const std::string&) const;
  virtual bool hasKey(const std::string&) const;
  virtual int findKey(const std::string&) const;
  virtual bool isSticky(int) const;
public: //IVisited
  virtual void* cast(const std::string&) const;
  virtual bool visit(Slash::Store::IConstVisitor&) const;
  virtual bool read(Slash::Store::IVisitor&);
public:
  Annotation();
  Annotation(const Annotation&);
  virtual ~Annotation();
  Annotation& operator=(const Annotation&);
  void clear();
  std::vector<Item>& items(); //for the IO.
  const std::vector<Item>& items() const; //for the IO.
private:
  Item* findItem(const std::string&);
  const Item* findItem(const std::string&) const;
  SLASH_STORE_HEADER
private:
  Vector<Item> fItems;
};

}

#endif
