/*
 * Example of writing a tuple in an hippodraw ascii file.
 */


#include <AIDA/IAnalysisFactory.h>
#include <AIDA/ITreeFactory.h>
#include <AIDA/ITree.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IHistogram2D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/ITupleFactory.h>
#include <AIDA/ITuple.h>

#include <Lib/Printer.h>
#include <Lib/Debug.h>
#include <Lib/Randoms.h>

#include <iostream>

//////////////////////////////////////////////////////////////////////////////
int main(
 int //aArgc
,char** //aArgv
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Debug::checkByClass(true);

  AIDA::IAnalysisFactory* analysisFactory = AIDA_createAnalysisFactory();
  if(!analysisFactory) {
    std::cout << "BatchLab_hippo::main :"
              << " Can't get an AnalysisFactory." << std::endl;
    return 1;
  }

  AIDA::ITreeFactory* treeFactory = analysisFactory->createTreeFactory();
  if(!treeFactory) {
    std::cout << "BatchLab_hippo::main :"
              << " Can't get a TreeFactory." << std::endl;
    return 1;
  }

  AIDA::ITree* tree = 
    treeFactory->create("BatchLab_hippo.tnt","hippodraw",false,true);
  if(!tree) {
    std::cout << "BatchLab_hippo::main :"
              << " Can't get a Tree." << std::endl;
    return 1;
  }

  if(!tree->mkdir("tuples")) {
    std::cout << "BatchLab_hippo::main :"
              << " can't make directory \"tuples\"." << std::endl;
    return 1;
  }

  if(!tree->cd("tuples")) {
    std::cout << "BatchLab_hippo::main :"
              << " can't cd to \"tuples\"." << std::endl;
    return 1;
  }

  // Get a tuple factory :
  AIDA::ITupleFactory* tupleFactory = 
    analysisFactory->createTupleFactory(*tree);
  if(!tupleFactory) {
    std::cout << "BatchLab_hippo::main :"
              << " Can't get a TupleFactory." << std::endl;
    return 1;
  }


  AIDA::ITuple* tuple = tupleFactory->create("tuple","my hippo tuple",
                                             "double rgauss, double rbw");
  if(!tuple) {
    std::cout << "BatchLab_hippo::main :"
              << " Can't create the \"tuple\" tuple." << std::endl;
    return 1;
  }

  Lib::RandomGauss randomGauss;
  Lib::RandomBreitWigner randomBreitWigner;
  for(int count=0;count<1000;count++) {
    tuple->fill(0,randomGauss.shoot());
    tuple->fill(1,randomBreitWigner.shoot());
    tuple->addRow();
  }

  if(!tree->commit()) {
    std::cout << "BatchLab_hippo::main :"
              << " can't commit." << std::endl;
    return 1;
  }

  delete tupleFactory;
  delete treeFactory;
  delete analysisFactory; // Will delete the trees.

  // A simple check to see if object creation/destruction balance is correct :
  Lib::Printer printer;
  Lib::Debug::balance(printer);

  return 0;
}

