#ifndef Lib_A3s_h
#define Lib_A3s_h

// Inheritance :
#include <Lib/Array.h>

#include <math.h> //sqrt

namespace Lib {

template <class T>
class A3 : public Array<T> {
public:
  A3():Array<T>(1,3){}
  A3(const A3& aFrom):Array<T>(aFrom){}
  A3(const T aVec[3]):Array<T>(1,3) {
    Array<T>::fVector[0] = aVec[0];
    Array<T>::fVector[1] = aVec[1];
    Array<T>::fVector[2] = aVec[2];
  }
  A3(const T& a0,const T& a1,const T& a2):Array<T>(1,3) {
    Array<T>::fVector[0] = a0;
    Array<T>::fVector[1] = a1;
    Array<T>::fVector[2] = a2;
  }
  virtual ~A3() {}
public:
  inline T value0() const { return Array<T>::fVector[0];}
  inline T value1() const { return Array<T>::fVector[1];}
  inline T value2() const { return Array<T>::fVector[2];}

  inline void setValue0(const T& aValue) { Array<T>::fVector[0] = aValue;}
  inline void setValue1(const T& aValue) { Array<T>::fVector[1] = aValue;}
  inline void setValue2(const T& aValue) { Array<T>::fVector[2] = aValue;}

  inline void setValue(const T& a1,const T& a2,const T& a3) {
    Array<T>::fVector[0] = a1;
    Array<T>::fVector[1] = a2;
    Array<T>::fVector[2] = a3;
  }
  inline void setValue(const T aV[3]) {
    Array<T>::fVector[0] = aV[0];
    Array<T>::fVector[1] = aV[1];
    Array<T>::fVector[2] = aV[2];
  }
  inline void value(T& a1,T& a2,T& a3) const {
    a1 = Array<T>::fVector[0];
    a2 = Array<T>::fVector[1];
    a3 = Array<T>::fVector[2];
  }    
};

#define A33_D22(a_11,a_12,a_21,a_22)  ((a_11) * (a_22) - (a_21) * (a_12))

template <class T>
class A33 : public ADD<T> {
public:
  A33():ADD<T>(3){}
  A33(const A33& aFrom):ADD<T>(aFrom){}
  virtual ~A33() {}
  inline A33& operator=(const A33& aFrom) {
    ADD<T>::operator=(aFrom);
    return *this;
  }

#define A33_SET_VALUE(aName,a1,a2) \
  inline void aName(const T& aValue) { Array<T>::fVector[a1 + a2 * 3] = aValue;}

  A33_SET_VALUE(setValue00,0,0)
  A33_SET_VALUE(setValue10,1,0)
  A33_SET_VALUE(setValue20,2,0)

  A33_SET_VALUE(setValue01,0,1)
  A33_SET_VALUE(setValue11,1,1)
  A33_SET_VALUE(setValue21,2,1)

  A33_SET_VALUE(setValue02,0,2)
  A33_SET_VALUE(setValue12,1,2)
  A33_SET_VALUE(setValue22,2,2)

#undef A33_SET_VALUE

#define A33_GET_VALUE(aName,a1,a2) \
  inline T aName() const { return Array<T>::fVector[a1 + a2 * 3];}

  A33_GET_VALUE(value00,0,0)
  A33_GET_VALUE(value10,1,0)
  A33_GET_VALUE(value20,2,0)

  A33_GET_VALUE(value01,0,1)
  A33_GET_VALUE(value11,1,1)
  A33_GET_VALUE(value21,2,1)

  A33_GET_VALUE(value02,0,2)
  A33_GET_VALUE(value12,1,2)
  A33_GET_VALUE(value22,2,2)

#undef A33_GET_VALUE

  inline virtual T determinant() const {
    T e00 =  A33_D22(value11(),value12(),value21(),value22());
    T e01 = -A33_D22(value01(),value02(),value21(),value22());
    T e02 =  A33_D22(value01(),value02(),value11(),value12());
    return (  value00() * e00
            + value10() * e01
            + value20() * e02);

  }

  inline bool invert(A33<T>& aResult) const {
    aResult.setValue00(
      A33_D22(value11(),value12(),
              value21(),value22()));
    aResult.setValue01(
      -A33_D22(value01(),value02(),
  	     value21(),value22()));
    aResult.setValue02(
      A33_D22(value01(),value02(),
              value11(),value12()));
                  
    double deter =
        value00() * aResult.value00()
      + value10() * aResult.value01()
      + value20() * aResult.value02();

    if(deter==Array<T>::zero()) return false;
                  
    aResult.setValue10(
     -A33_D22(value10(),value12(),
              value20(),value22()));
    aResult.setValue11(
      A33_D22(value00(),value02(),
  	    value20(),value22()));
    aResult.setValue12(
     -A33_D22(value00(),value02(),
              value10(),value12()));
                    
    aResult.setValue20(
      A33_D22(value10(),value11(),
  	    value20(),value21()));
    aResult.setValue21(
     -A33_D22(value00(),value01(),
  	    value20(),value21()));
    aResult.setValue22(
      A33_D22(value00(),value01(),
  	    value10(),value11()));
                    
    aResult.multiply(1./deter);  
  
    return true;
  }
#undef A33_D22

};

template <class T>
class A333 : public Array<T> {
public:
  A333():Array<T>(3,3){}
  A333(const A333& aFrom):Array<T>(aFrom){}
  virtual ~A333() {}

  inline T value(unsigned int a1,unsigned int a2,unsigned int a3) const { 
    return Array<T>::fVector[a1 + a2 * 3 + a3 * 9];
  }

  inline void setValue(unsigned int a1,unsigned int a2,unsigned int a3,const T& aValue) { 
    Array<T>::fVector[a1 + a2 * 3 + a3 * 9] = aValue;
  }

#define A333_SET_VALUE(aName,a1,a2,a3) \
  inline void aName(const T& aValue) { Array<T>::fVector[a1 + a2 * 3 + a3 * 9] = aValue;}

  A333_SET_VALUE(setValue000,0,0,0)
  A333_SET_VALUE(setValue100,1,0,0)
  A333_SET_VALUE(setValue200,2,0,0)
  A333_SET_VALUE(setValue010,0,1,0)
  A333_SET_VALUE(setValue110,1,1,0)
  A333_SET_VALUE(setValue210,2,1,0)
  A333_SET_VALUE(setValue020,0,2,0)
  A333_SET_VALUE(setValue120,1,2,0)
  A333_SET_VALUE(setValue220,2,2,0)

  A333_SET_VALUE(setValue001,0,0,1)
  A333_SET_VALUE(setValue101,1,0,1)
  A333_SET_VALUE(setValue201,2,0,1)
  A333_SET_VALUE(setValue011,0,1,1)
  A333_SET_VALUE(setValue111,1,1,1)
  A333_SET_VALUE(setValue211,2,1,1)
  A333_SET_VALUE(setValue021,0,2,1)
  A333_SET_VALUE(setValue121,1,2,1)
  A333_SET_VALUE(setValue221,2,2,1)

  A333_SET_VALUE(setValue002,0,0,2)
  A333_SET_VALUE(setValue102,1,0,2)
  A333_SET_VALUE(setValue202,2,0,2)
  A333_SET_VALUE(setValue012,0,1,2)
  A333_SET_VALUE(setValue112,1,1,2)
  A333_SET_VALUE(setValue212,2,1,2)
  A333_SET_VALUE(setValue022,0,2,2)
  A333_SET_VALUE(setValue122,1,2,2)
  A333_SET_VALUE(setValue222,2,2,2)

#undef A333_SET_VALUE

};

class A3d : public A3<double> {
public:
  A3d();
  A3d(const double aVec[3]);
  A3d(const double&,const double&,const double&);
  virtual ~A3d() {}
public:
  inline double length() const {
    return ::sqrt(fVector[0] * fVector[0] + 
                  fVector[1] * fVector[1] + 
                  fVector[2] * fVector[2]); 
  }
  inline double normalize() {
    double norme = length();
    if(norme==0) return 0;
    divide(norme);
    return norme;
  }
  inline Lib::A3d cross(const Lib::A3d& aV) const {
    return A3d(fVector[1] * aV.fVector[2] - fVector[2] * aV.fVector[1],
               fVector[2] * aV.fVector[0] - fVector[0] * aV.fVector[2],
               fVector[0] * aV.fVector[1] - fVector[1] * aV.fVector[0]);
  }    
  inline double dot(const Lib::A3d& aV) const {
    return (fVector[0] * aV.fVector[0] + 
            fVector[1] * aV.fVector[1] + 
            fVector[2] * aV.fVector[2]);
  }    
};

}

template <class T>
Lib::A33<T> operator-(const Lib::A33<T>& a1,const Lib::A33<T>& a2) {
  Lib::A33<T> res(a1);
  res.subtract(a2);
  return res;
}
template <class T>
Lib::A33<T> operator+(const Lib::A33<T>& a1,const Lib::A33<T>& a2) {
  Lib::A33<T> res(a1);
  res.add(a2);
  return res;
}
template <class T>
Lib::A33<T> operator*(const Lib::A33<T>& a1,const Lib::A33<T>& a2) {
  Lib::A33<T> res;
  a1.mxmul(a2,res);
  return res;
}

namespace Lib {

template <class T>
A33<T> commutator(const A33<T>& a1,const A33<T>& a2) {
  return a1*a2-a2*a1;
}
template <class T>
A33<T> antiCommutator(const A33<T>& a1,const A33<T>& a2) {
  return a1*a2+a2*a1;
}

//////////////////////////////////////////////////////////////////////////////
/// Some common real matrices ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
typedef A33<double> A33d;
typedef A333<double> A333d;

//  1  0  0
//  0  1  0
//  0  0  1
class A33d_I : public A33d {
public:
 A33d_I();
};

// Generators og rotation group :
// Rk(i,j) = epsilon(k,i,j)  i,j,k=1,2,3
//  0  0  0
//  0  0  1
//  0 -1  0
class A33d_R1 : public A33d {
public:
 A33d_R1();
};
//  0  0 -1
//  0  0  0
//  1  0  0
class A33d_R2 : public A33d {
public:
 A33d_R2();
};
//  0  1  0
// -1  0  0
//  0  0  0
class A33d_R3 : public A33d {
public:
 A33d_R3();
};

// Levi-Civita indicator :
class A333d_epsilon : public A333d {
public:
 A333d_epsilon();
};

}

#endif


