#ifndef Lib_As_h
#define Lib_As_h

// Inheritance :
#include <Lib/Array.h>

#include <complex>

namespace Lib {

class ADDd_E : public ADD<double> {
public:
  static bool groupConstants(unsigned int,Lib::Array<double>&);
  static bool metric(unsigned int,Lib::ADD<double>&);
  static bool cartanMetric(unsigned int,Lib::ADD<double>&);
  static bool casimir(unsigned int,Lib::ADD<double>&);
public:
  ADDd_E(unsigned int aOrder,unsigned int aIndex);
  virtual ~ADDd_E();
private:
  bool initialize(unsigned int,unsigned int);
};

template <class T>
class A666 : public Array<T> {
public:
  A666():Array<T>(3,6){}
  A666(const A666& aFrom):Array<T>(aFrom){}
  virtual ~A666() {}
  inline T value(unsigned int a1,unsigned int a2,unsigned int a3) const { 
    return Array<T>::fVector[a1 + a2 * 6 + a3 * 36];
  }
  inline void setValue(unsigned int a1,unsigned int a2,unsigned int a3,const T& aValue) { 
    Array<T>::fVector[a1 + a2 * 6 + a3 * 36] = aValue;
  }
};

class A666d_LorentzGroupConstants : public A666<double> {
public:
 A666d_LorentzGroupConstants();
};

class A66d_LorentzCartanMetric : public ADD<double> {
public:
 A66d_LorentzCartanMetric();
};

class A66d_LorentzVecMetric : public ADD<double> {
public:
 A66d_LorentzVecMetric();
};

class A66z_LorentzDiracMetric : public ADD< std::complex<double> > {
public:
 A66z_LorentzDiracMetric();
};

class A66z_LorentzWeylMetric : public ADD< std::complex<double> > {
public:
 A66z_LorentzWeylMetric();
};

class ADDd_PoincareCartanMetric : public ADD<double> {
public:
 ADDd_PoincareCartanMetric(unsigned int);
};

template <class T>
class Kronecker : public Array<T> {
public:
  Kronecker(unsigned int aOrder):Array<T>(aOrder,aOrder)
  ////////////////////////////////////////////////////////////////////////////
  //  epsilon(i1,i2,....in) with n = aOrder and i in [0,n[.
  // Then an Array of aOrder * aOrder.
  //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
  {
    unsigned int index = 0;
    typedef typename std::vector<unsigned int> UInts;
    UInts is(aOrder);
    std::vector<T>& vec = Array<T>::vector();
    typedef typename std::vector<T>::iterator VecIt;
    VecIt it = vec.begin();
    for(;it!=vec.end();++it,index++) {
      if(!Array<T>::indices(index,is)) return; //FIXME throw.
      bool good = true;
     {for(unsigned iaxis=0;iaxis<aOrder;iaxis++) {
        unsigned int ival = is[iaxis];
        for(unsigned iaxis2=iaxis+1;iaxis2<aOrder;iaxis2++) {
          if(is[iaxis2]==ival) {
            good = false;
            break;
          }
        }
        if(!good) break;
      }}
      if(!good) continue;
      // All indicies are different.
      unsigned int n = 0;
      for(unsigned iaxis=0;iaxis<aOrder;) { 
        unsigned int ival = is[iaxis];
        if(ival!=iaxis) {
          // Swap and add one permutation :
          unsigned int old = is[ival];
          is[ival] = ival;
          is[iaxis] = old;
          n +=1;
        } else {
          iaxis++;
        }
      }
     {unsigned int n_2 = n/2;
      if(2*n_2==n) (*it) = Array<T>::one();
      else (*it) = Array<T>::minus_one();}
    }
  }
};

template <class T>
ADD< std::complex<double> > tocomplex(const ADD<T>& aFrom) {
  unsigned int order = aFrom.order();
  typedef std::complex<double> z;
  ADD<z> to(order);
  const std::vector<T>& fvec = aFrom.vector();
  std::vector<z>& tvec = to.vector();
  //Array<T>::ConsVecIt fit = fvec.begin(); //FIXME : Does not compile.
  typedef typename std::vector<T>::const_iterator ConsVecIt;
  ConsVecIt fit = fvec.begin();
  //Array< std::complex<double> >::VecIt tit = tvec.begin();
  typedef typename std::vector<z>::iterator VecIt;
  VecIt tit = tvec.begin();
  for(;fit!=fvec.end();++fit,++tit) *tit = *fit;
  return to;
}

//////////////////////////////////////////////////////////////////////////////
template <class T>
bool determinant(const Lib::ADD<T>& aMatrix,T& aDeterminant) 
//////////////////////////////////////////////////////////////////////////////
// Brut force determinant by using the kroneckers.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  aDeterminant = aMatrix.zero();

  unsigned int order = aMatrix.order(); 

  typedef typename std::vector<unsigned int> UInts;
  typedef typename std::vector<T>::const_iterator ConsVecIt;

  Kronecker<T> k_c(order);
  const std::vector<T>& vec_c = k_c.vector();
  ConsVecIt it_c;
  UInts is_c(order);

  //Kronecker<T> k_r(order);
  //const std::vector<T>& vec_r = k_r.vector();
  //ConsVecIt it_r;
  //UInts is_r(order);

  unsigned int index_c = 0;
  for(it_c = vec_c.begin();it_c!=vec_c.end();++it_c,index_c++) {
    if(*it_c==0.) continue;
    if(!k_c.indices(index_c,is_c)) return false;
    //unsigned int index_r = 0;
    //for(it_r = vec_r.begin();it_r!=vec_r.end();++it_r,index_r++) {
    //  if(*it_r==0.) continue;
    //  if(!k_r.indices(index_r,is_r)) return false;
      T v = Array<T>::one();
      for(unsigned iorder=0;iorder<order;iorder++) {
        //v *= aMatrix.value(is_c[iorder],is_r[iorder]); 
        v *= aMatrix.value(is_c[iorder],iorder); 
      }
      aDeterminant += v;
    //}
  }

  //{T fact = Array<T>::one();
  //for(unsigned index=1;index<=order;index++) fact *= index;
  //aDeterminant /= fact;}

  return true;
}

}

#endif


