#ifndef OnX_BaseUI_h
#define OnX_BaseUI_h 

// Inheritance :
#include <Slash/UI/IUI.h>
#include <OnX/Interfaces/IIB.h>

#include <OnX/Core/CallbackData.h>
#include <OnX/Core/WidgetFactory.h>
#include <OnX/Core/Action.h> //FIXME NativeWidget

#include <list>

namespace Slash {namespace Core {class IWriter;}}
namespace Slash {namespace UI {class IScriptManager;}}
namespace Slash {namespace UI {class IWidgetFactory;}}

namespace Lib {class ItemML;}
namespace Slash {namespace Core {class IRelationManager;}}

namespace OnX {

//FIXME class Action;
class BaseUI;

class Execute {
public:
  Execute(BaseUI&,const std::string&);
  const std::string& what() const;
  BaseUI& baseUI();
private:
  BaseUI& fUI;
  std::string fWhat;
};

class BaseUI 
:public virtual Slash::UI::IUI
,public virtual IIB 
{
public: //Slash::UI::IUI
  virtual void* cast(const std::string&) const;
  virtual bool isValid() const;
  virtual int steer();
  virtual void executeScript(const std::string&,const std::string&);
  virtual void addDispatcher(Slash::UI::IUI::Dispatcher);
  virtual bool setCallback(const std::string&,
                           const std::string&,
                           const std::string&,
                           const std::string&,
                           int = 0);
  virtual bool removeCallback(const std::string&,
                              const std::string&,
                              int = 0);
  virtual std::string callbackValue();
  virtual const Slash::UI::ICallbackData& callbackData();
  virtual bool setCallbackData(const Slash::UI::ICallbackData&);
  virtual bool executeCallbacks(const std::string&,const std::string&);
  virtual bool load(const std::string&,const std::string&,bool = true);
  virtual bool save();
  virtual void deleteChildren(const std::string&);
  virtual void echo(const std::string&);
  virtual std::string ask(const std::string&);
  virtual Slash::UI::IWidget* findWidget(const std::string&);
  virtual Slash::UI::IWidget* createWidget(const Slash::UI::IWidgetClass&,
                                const std::string&,
                                Slash::UI::IWidget&,
                                const std::vector<std::string>& = std::vector<std::string>());
  virtual bool size(const Slash::UI::IWidget&,unsigned int&,unsigned int&);
  virtual bool createComponent(const std::string&,
                               const std::string&,
                               const std::string&);
  virtual bool destroyComponent(const std::string&);
  virtual bool copyComponent(const std::string&,
                             const std::string&,
                             const std::string&);
  virtual bool size(const std::string&,unsigned int&,unsigned int&);
  virtual std::vector<std::string> findChildren(const std::string&,
                                                const std::string&,
                                                const std::string&) const;
  virtual Slash::Core::ISession& session();
  virtual Slash::UI::ICyclic* findCyclic(const std::string&) const;
  virtual bool isComponent(const std::string&);
  virtual int steerLevel() const;

  virtual bool beingDestroyed() const;
  virtual bool finalize();

  //FIXME : deprecated
  //virtual std::string currentViewer();
  //virtual bool setCurrentViewer(const std::string&);
  virtual Slash::UI::IWidget* currentWidget() const; 
  virtual bool setCurrentWidget(Slash::UI::IWidget*); 

  virtual bool enableTerminal(const std::string&,const std::string&);
  virtual bool disableTerminal();
  virtual bool redirectAskToTerminal(bool); 
  virtual bool lockScreen(const std::string& passwd,
                          const std::string& interp = "",
                          const std::string& script = ""); 
public: //IIB  
  virtual std::string tree() const;
  virtual bool loadReferenceFile(Lib::ItemML&, const std::string&);
  virtual Lib::ItemML* topItemML() const;
  virtual Slash::UI::IWidgetClass* findWidgetClass(const std::string&) const;
  virtual std::vector< std::pair<bool,std::string> > parseDecorations(const std::string&) const;
  virtual const std::vector<std::string>& callbackLanguageList() const;
  virtual bool propertyType(const std::string&,std::string&) const;
  virtual std::string createWidgetName();
public:
  // Used in QtUI :
  void setCallbackData(const CallbackData&);
  bool notification(Slash::Core::IMessage*);
  // Used in Hippo :
  Lib::ItemMLFactory& itemMLFactory();
  void addFactory(Slash::UI::IWidgetFactory*);
  Slash::Core::IWriter& printer() const;
  static bool executeItemMLCallbacks(Lib::ItemML&,void*);
  //
  void addWidgetClass(Slash::UI::IWidgetClass*);
protected:
  BaseUI(Slash::Core::ISession&);
  virtual ~BaseUI();
protected:
  // REVIST : should have some IDevUI.h for the below.
  virtual int nativeSteer() = 0;
  virtual bool nativeSize(void*,unsigned int&,unsigned int&) = 0;
  virtual bool createNativeWidget(Lib::ItemML&,
                            const std::string&, //class
                            const std::string&, //name
                            Lib::ItemML* //parent
                            ) = 0;
  virtual bool destroyWidget(Lib::ItemML&) = 0;
  virtual Action* addCallback(Lib::ItemML&,
                              const std::string&,
                              const std::string&,
                              const std::string&) = 0;
  virtual void deleteCallback(Lib::ItemML&,
                              const std::string&,
                              const Action&) = 0;
  virtual std::string widgetName(Lib::ItemML&) = 0;
  virtual void showAll(Lib::ItemML&) = 0;
  //
  virtual Action* getAction(OnX::Action::NativeWidget);
  virtual Action* getAction(const std::string&);
  virtual Action* getAction(int);
  virtual Action* getAction(OnX::Action::NativeWidget,const std::string&,int = 0);
  //
  bool startup(const std::string&);
  std::list<Action> fActions;
  Action& addAction(Slash::UI::IUI&,
                    Slash::UI::IWidget*,
                    OnX::Action::NativeWidget,
                    const std::string&,
                    const std::string&,
                    const std::string& = "",int = 0);
  void removeActions(OnX::Action::NativeWidget);
  void removeAction(OnX::Action::NativeWidget,const std::string&,int = 0);
  void removeDispatcher(Slash::UI::IUI::Dispatcher);
  bool toWidgetAndResource(const std::string&,std::string&,std::string&);
  bool propertyValue(const std::string&,const std::string&,std::string&);
  virtual void* findNativeWidget(const std::string&);
  virtual Lib::ItemML* createComponent(const std::string&,
                                       const std::string&,
                                       Lib::ItemML&);
  virtual bool solveInputDialog(Lib::ItemML&);
  virtual bool createInputDialog(Lib::ItemML&);
  virtual std::string buildName(Lib::ItemML&);
  virtual Lib::ItemML* parentItemML(Lib::ItemML&);
  virtual bool searchChild(Lib::ItemML&,
                           const std::string&,
                           const std::string&,
                           std::vector<std::string>&) const;
  virtual void steerModal();
  bool createIf(Lib::ItemML&);
  Lib::ItemML* findFirstWidget(Lib::ItemML&);
  std::vector<std::string> disableCallbackScript(OnX::Action::NativeWidget,const std::string&);
  bool enableCallbackScript(OnX::Action::NativeWidget,const std::string&,const std::vector<std::string>&);
  bool createChildren(Lib::ItemML&);
private:
  bool executeCallbacks(Lib::ItemML&,const std::string&);
  bool createNativeWidget(Lib::ItemML&);
  std::string subTree(Lib::ItemML&,int) const;
  void createWidgetTable();
  void createLanguageList();
  void createPropertyList();
protected:
  Slash::Core::ISession& fSession;
  std::string fName;
  CallbackData fCallbackData;
  std::list<Slash::UI::IUI::Dispatcher> fDispatchers;
  Lib::ItemML* fTopItemML;
  Slash::UI::IScriptManager* fScriptManager;
  std::list<Slash::UI::ICyclic*> fCyclics;
  int fWidgetNameCounter;
  bool fBeingDestroyed;
  int fVerboseLevel;
  int fSteerLevel;
  Slash::Core::IRelationManager* fRelationManager;
  Slash::UI::IWidget* fCurrentWidget;
  bool fAskToTerminal;
private: //IB
  std::vector<Slash::UI::IWidgetClass*> fWidgetClasses;
  std::vector<std::string> fCallbackLanguageList;
  std::vector<std::string> fPropertyList;
  WidgetFactory fWidgetFactory;
  std::vector<Slash::UI::IWidgetFactory*> fFactories;
};

}

#endif


