//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,const std::vector<std::string>&);
//   }
//

// Slash :
#include <Slash/UI/IUI.h>

// Lib : helpers only, no link dependency :
#include <Lib/Cast.h>
#include <Lib/Manager.h>

// OnXTemplateFive :
#include <OnXTemplateFive/Interfaces/IUserManager.h>

namespace OnXTemplateFive {

class UserManager : public virtual IUserManager {
public: //Slash::Core::IManager
  virtual std::string name() const { return fName;}
  virtual void* cast(const std::string& aClass) const { //Safe cast.
    if_Lib_SCast(OnXTemplateFive::UserManager)
    else if_Lib_SCast(Slash::Core::IManager)
    else if_Lib_SCast(OnXTemplateFive::IUserManager)
    else return 0;
  }
public: //IUserManager
  virtual void userMethod() {
    // Our method :
    Slash::Core::IWriter& printer = fSession.printer();
    printer.write("From OnXTemplateFive::UserManager::userMethod() !\n");
  }
public:
  UserManager(Slash::Core::ISession& aSession)
  :fSession(aSession),fName("OnXTemplateFive::UserManager"){}
  virtual ~UserManager(){}
private:
  Slash::Core::ISession& fSession;
  std::string fName; //Name of our manager.
};

}

extern "C" {

//////////////////////////////////////////////////////////////////////////////
void OnXTemplateFive_initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Should be executed once. 
// Here done on the create callback of the :
//   scripts/OnX/OnXTemplateFive.onx 
// file loaded by onx.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  OnXTemplateFive::UserManager* manager = new OnXTemplateFive::UserManager(session);
  session.addManager(manager);
}
//////////////////////////////////////////////////////////////////////////////
void OnXTemplateFive_userMethod(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Executed from "File/User manager method" menu item.
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  // Find our manager in the session.
  // We use the Lib_findManager helper that uses the safe cast.
  OnXTemplateFive::IUserManager* userManager = 
    Lib_findManager(aUI.session(),
      "OnXTemplateFive::UserManager",OnXTemplateFive::IUserManager);
  if(!userManager) {
    aUI.echo("OnXTemplateFive::UserManager not found.");
    return;
  }

  // Call our manager method :
  userManager->userMethod();
}

} // extern "C"
